package edu.illinois.library.cantaloupe.operation;

import edu.illinois.library.cantaloupe.test.BaseTest;
import org.junit.Before;
import org.junit.Test;

import java.awt.Dimension;
import java.util.Map;

import static org.junit.Assert.*;

public class SharpenTest extends BaseTest {

    private Sharpen instance;

    @Before
    public void setUp() throws Exception {
        super.setUp();

        this.instance = new Sharpen();
    }

    @Test
    public void getResultingSize() {
        Dimension fullSize = new Dimension(200, 200);
        assertEquals(fullSize, instance.getResultingSize(fullSize));
    }

    @Test
    public void hasEffect() {
        assertFalse(instance.hasEffect());
        instance.setAmount(0.1f);
        assertTrue(instance.hasEffect());
    }

    @Test
    public void hasEffectWithArguments() {
        Dimension fullSize = new Dimension(300, 200);
        OperationList opList = new OperationList(new Crop(0, 0, 300, 200));

        assertFalse(instance.hasEffect(fullSize, opList));
        instance.setAmount(0.1f);
        assertTrue(instance.hasEffect(fullSize, opList));
    }

    @Test
    public void setAmount() {
        final float delta = 0.00000001f;
        instance.setAmount(0f);
        assertEquals(0f, instance.getAmount(), delta);
        instance.setAmount(0.1f);
        assertEquals(0.1f, instance.getAmount(), delta);
    }

    @Test(expected = IllegalArgumentException.class)
    public void setAmountToIllegalValue() {
        instance.setAmount(-0.1f);
    }

    @Test(expected = IllegalStateException.class)
    public void setAmountWhenFrozenThrowsException() {
        instance.freeze();
        instance.setAmount(0.1f);
    }

    @Test
    public void toMap() {
        Map<String,Object> map = instance.toMap(new Dimension(0, 0));
        assertEquals(instance.getClass().getSimpleName(), map.get("class"));
        assertEquals(0f, map.get("amount"));

        instance.setAmount(0.2f);
        map = instance.toMap(new Dimension(0, 0));
        assertEquals(0.2f, map.get("amount"));
    }

    @Test
    public void toMapReturnsUnmodifiableMap() {
        Dimension fullSize = new Dimension(100, 100);
        Map<String,Object> map = instance.toMap(fullSize);
        try {
            map.put("test", "test");
            fail("Expected exception");
        } catch (UnsupportedOperationException e) {
            // pass
        }
    }

    @Test
    public void testToString() {
        assertEquals("0.0", instance.toString());
        instance.setAmount(0.2f);
        assertEquals("0.2", instance.toString());
    }

}
